; Macros definition : 
;
; TimeOut		Operations to do once the timeout occurs (Probably a call)
; UpdateTimer	Operations to do once the Timer needs an update
;
; Procedures definition : 
;
; void LoadInt();
; void NewInt();
; Byte FadeOut();
; Byte FadeIn(Byte Max);

; Procedure to load the interrupt and it's table
; [In]   : Null
; [Out]  : Null
; [Uses] : a, hl, bc, de
LoadInt:
	ld   hl, NewInt
	ld   de, IntDest
	ld   bc, IntEnd - NewInt
	ldir												; Copy our interrupt to Int. RAM

	ld   a, IntDest / 100h
	ld   hl, IntTab
	ld   (hl), a
	ld   de, IntTab + 1
	ld   bc, 100h
	ldir												; Install the interrupt table

	ld   a, IntTab / 100h
	ld   i, a
SetInt:
	im   2
	set  2, (iy + 23h)
	ei
	ret


; Here lies our new interrupt ([104] Must not exceed 104bytes)
; It's responsible for the Short and Long timer and the Greyscale
; [In] = [Out] = [Uses] = af'
;<[]>---<[]>---<[]>---<[]>---<[]>---<[]>---<[]>---<[]>---<[]>---<[]>---<[]>---<[]>---<[]>---<[]>
NewInt:
	ex   af, af'

#ifdef ShortT
STimer	 equ (IntDest + $ - NewInt) + 1
	ld   a, 0
	dec  a
	ld   (STimer), a									; Update the short timer
#endif
#ifdef LongT
Timer	 equ (IntDest + $ - NewInt) + 1
	ld   a, IntSpd
	dec  a
	ld   (Timer), a										; Update the long timer
	jr   nz, NoUpdate

	push hl
	ld   a, IntSpd
	ld   (Timer), a
	ld   a, (SecLeft)
	dec  a
	ld   (SecLeft), a
	jp   p, (UpdateMS - NewInt) + IntDest

	ld   a, 59
	ld   (SecLeft), a
	ld   a, (MinLeft)
	dec  a
	ld   (MinLeft), a				; If we have a carry over here, it's game over ya know :)?

	ld   hl, LevOut
	inc  a
	jr   z, GameOver

UpdateMS:
	ld   hl, UpdateTimer
GameOver:
	ld   (ShowPtch), hl
	pop  hl
#endif

NoUpdate:
	in   a, (3)
	bit  1, a
	jr   z, EndInt										; VBL Check

CurPage	 equ (IntDest + $ - NewInt) + 1
	ld   a, 3											; Code patching done here!
	dec  a
	jr   nz, SetScr2

	ld   a, 3
	ld   (CurPage), a
	ld   a, 0Ah
	out  (0), a

	jr   EndInt

SetScr2:
	ld   (CurPage), a
	ld   a, 3Ch
	out  (0), a											; Set the video page!

EndInt:
	in   a, (3)
	rra
	ld   a, 0
	adc  a, 9
	out  (3), a
	ld   a, 0Bh
	out  (3), a

	ex   af, af'
	ei
	reti
IntEnd:
MinLeft	 equ (IntDest + $ - NewInt)
SecLeft	 equ (IntDest + $ - NewInt) + 1

#ifdef ShortT
#ifdef Fades
; Procedure to fade the screen from normal to white
; [In] = 						[Out] : a = Old contrast			[Uses] : af, bc, hl
;<[]>---<[]>---<[]>---<[]>---<[]>---<[]>---<[]>---<[]>---<[]>---<[]>---<[]>---<[]>---<[]>---<[]>
FadeOut:
	ld   hl, STimer
	ld   b, 0FFh
	ld   a, (0C008h)
	push af
	ld   c, a

LitMore:
	ld   (hl), b
LitDelay:
	bit  4, (hl)
	jr   nz, LitDelay

	dec  a
	out  (2), a
	jr   nz, LitMore

	pop  af
	ret


; Procedure to fade the screen from white to normal
; [In] : [MaxCont]	[Out] = NULL 				[Uses] = af, bc
;<[]>---<[]>---<[]>---<[]>---<[]>---<[]>---<[]>---<[]>---<[]>---<[]>---<[]>---<[]>---<[]>---<[]>
FadeIn:
	ld   hl, STimer
	xor  a
	ld   b, 0FFh

DimMore:
	ld   (hl), b
DimDelay:
	bit  4, (hl)
	jr   nz, DimDelay

	inc  a
	out  (2), a
MaxCont  equ $ + 1
	cp   0
	jr   nz, DimMore

	ret
#endif
#endif
